import glob
import os.path
import pathlib
from typing import List, Dict, Union

import cv2
import numpy as np
import pandas as pd
import pydicom
from pydicom.errors import InvalidDicomError
from scipy import interpolate

Path = Union[str, os.PathLike]


def __always_true(*args, **kwargs) -> bool: return True


def _to_str(i):
    if not 99 >= i >= 0 or not round(i) == i:
        raise ValueError(f"i should be 2 digits, but instead it is {i}")
    return str(i) if i >= 10 else f"0{i}"


def get_log(log: str, root: str):
    if log is None:
        log = os.path.join(root, "val_contour_log.csv")
    if os.path.isfile(log):
        df = pd.read_csv(log, index_col=0)
    else:
        df = pd.DataFrame(columns=["ID", "contour_type", "result", "by", "frame", "slice", "remark", "time_stamp"])
    return df


def _has_dicom(pth: Path) -> bool:
    for f in glob.glob(os.path.join(pth, "*")):
        try:
            pydicom.dcmread(f)
            return True
        except InvalidDicomError:
            continue
        except PermissionError:
            continue
    return False


def get_seq_paths(root: Path) -> List[str]:
    """
    checks for sequences, which are assumed to  be 3 levels into the root;

    Args:
        root: path to the Horos Dataset
    Returns:
        list of sequence names
    """
    return [pth for pth in globSSF("*/*/*/", root_dir=root) if _has_dicom(os.path.join(root, pth))]


def get_ids(root: Path):
    """
    gets a list of IDs; not as trivial, as there can be other folders, too;
    checks for sequences, which are assumed to  be 3 levels into the root;
    splits the first directory of it and returns non duplicates of it
    Args:
        root: path to the Horos Dataset
    Returns:
        list of IDs
    """
    seqs = get_seq_paths(root)
    result = [pathlib.Path(seq).parts[0] for seq in seqs]
    return sorted(list(set(result)))


def get_log_path(log: str, root: str):
    if log is None:
        log = os.path.join(root, "val_contour_log.csv")
    return log


def globSSF(path_name, /, root_dir, **kwargs) -> List[str]:
    """mimcks behaviour of future globs"""
    return [os.path.relpath(pth, root_dir) for pth in
            glob.glob(os.path.normpath(os.path.join(root_dir, path_name)), **kwargs)]


def mask_from_omega_contour(cines: np.ndarray, contours: Dict[str, np.ndarray], f: int, s: int) -> np.ndarray:
    mask = np.zeros_like(cines[f, s].pixel_array)
    for i, (name, c) in enumerate(contours.items()):
        tck, _ = interpolate.splprep([*zip(*c[f, s])], s=0, per=True)
        xnew, ynew = interpolate.splev(np.linspace(0, 1, 500), tck, der=0)
        smooth_contour = np.array([(round(px), round(py)) for px, py in zip(xnew, ynew)])
        mask = cv2.drawContours(mask, [smooth_contour], 0, i + 1, - 1)
    return mask




import os.path

import pandas as pd
import pytest


@pytest.fixture()
# if testing horos; with our data, change this path to where your data is located
def horos_test_root():
    return os.path.normpath("./tests/horos_dummy")


@pytest.fixture()
def horos_test_seq_paths(horos_test_root):
    seq_paths = ['Impression_Cmr0064\\Anonymous_Study - 0\\CINE_segmented_LAX_2Ch_7\\',
                 'Impression_Cmr0064\\Anonymous_Study - 0\\CINE_segmented_LAX_3Ch_5\\',
                 'Impression_Cmr0064\\Anonymous_Study - 0\\CINE_segmented_LAX_4Ch_6\\',
                 'Impression_Cmr0064\\Anonymous_Study - 0\\CINE_segmented_SAX_28\\',
                 'Impression_Cmr0067\\Anonymous_Study - 0\\CINE_segmented_LAX_2Ch_7\\',
                 'Impression_Cmr0067\\Anonymous_Study - 0\\CINE_segmented_LAX_3Ch_9\\',
                 'Impression_Cmr0067\\Anonymous_Study - 0\\CINE_segmented_LAX_4Ch_10\\',
                 'Impression_Cmr0067\\Anonymous_Study - 0\\CINE_segmented_SAX_53\\']

    return [os.path.normpath(os.path.join(horos_test_root, seq_path)) for seq_path in seq_paths]


@pytest.fixture()
def horos_test_seq_n_slices():
    return [1, 1, 1, 13, 1, 1, 1, 13]


@pytest.fixture()
def horos_basal_first_file(horos_test_root):
    return os.path.join(horos_test_root, "basal_info.csv")


@pytest.fixture()
def horos_contour_info_path(horos_test_root):
    return os.path.join(horos_test_root, "target_contour_info.csv")


@pytest.fixture()
def horos_image_info_path(horos_test_root):
    return os.path.join(horos_test_root, "target_image_info.csv")


@pytest.fixture()
def log_dummy():
    return pd.read_csv("./tests/visual_confirmation.csv", index_col=0)


def pytest_addoption(parser):
    parser.addoption("--pth")




slice_types = ("cine_sa", "cine_2ch", "cine_3ch", "cine_4ch")

contour_types = (
    "sax_lv_endo",
    "sax_lv_epi",
    "sax_rv",
    "lax_2ch_lv_wall",
    "lax_2ch_lv_vol",
    "lax_2ch_la",
    "lax_3ch_lv_wall",
    "lax_3ch_lv_vol",
    "lax_3ch_rv",
    "lax_4ch_lv_wall",
    "lax_4ch_lv_vol",
    "lax_4ch_rv",
    "lax_4ch_la",
    "lax_4ch_ra",
    "omega_4ch"
)

omega_types = ("omega_4ch", "omega_3ch", "omega_2ch", "omega_sax")

#TODO: omega_3ch_names, omega_2ch_names, omega_sax_names
omega_4ch_names = ("rv_vol", "lv_vol", "ra", "la", "lv_wall")  # now also denotes plotting order

time_format = "%Y/%m/%d, %H:%M:%S"



import pytest

from horos_io import validation


@pytest.mark.parametrize("crit, target", [({"ID": "Impression_Cmr0010", "frame": 10, "slice": 0, "contour_type": "omega_4ch"}, True),
                                          ({"ID": "Impression_Cmr0210", "frame": 10, "slice": 0, "contour_type": "omega_4ch"}, False)])
def test_last_validation_was_successful2(log_dummy, crit, target):
    assert validation.last_validation_was_successful(log_dummy, conf_field="result", **crit) == target




from horos_io import core


def test_load_horos_contour_unstructured():
    """might need to extend this"""
    pth = "tests/omega_4ch.xml"
    result = core.load_horos_contour_unstructured(pth)
    assert len(result) == 10
